@@ -39,7 +39,7 @@ page()
 	if (argcnt > optind) {
 		mprintf(head_page);
 		for (i = optind; i < argcnt; i++) {
-			addr = getaddr(args[i]);
+			addr = getvalue(args[i]);
 			addr = print_page(addr);
 		}
 	} else {
@@ -74,7 +74,7 @@ vm_area_struct()
 		THROW(usage);
 	}
 	while (args[optind]) {
-		(void)print_vm_area_struct(getaddr(args[optind]));
+		(void)print_vm_area_struct(getvalue(args[optind]));
 		optind++;
 	}
 	return 0;
