@@ -144,7 +144,7 @@ print_usb_driver(addr, list_offset, nflag)
 			ud.id_table);
 	}
 	memread((addr_t)ud.name, sizeof(buf), buf, "name");
-	mprint_str(buf, sizeof(buf));
+	mprintstr(buf, sizeof(buf));
 	mprintf("\n");
 
 	return (addr_t)ud.driver_list.next;
