@@ -110,11 +110,9 @@ print_task(addr, full)
 	case TASK_RUNNING:		c = 'R';	break;
 	case TASK_INTERRUPTIBLE:	c = 'I';	break;
 	case TASK_UNINTERRUPTIBLE:	c = 'U';	break;
-	case TASK_ZOMBIE:		c = 'Z';	break;
 	case TASK_STOPPED:		c = 'T';	break;
-#ifdef TASK_SWAPPING
-	case TASK_SWAPPING:		c = 'S';	break;
-#endif /*TASK_SWAPPING*/
+	case TASK_ZOMBIE:		c = 'Z';	break;
+	case TASK_DEAD:			c = 'D';	break;
 	default:			c = '?';	break;
 	}
 	mprintf("%c %5d %5d %5d %5d ", c, ts.pid, ts.session, ts.uid, ts.euid);
