@@ -44,6 +44,7 @@ struct sizetbl sizetbl[] = {
 	{ "dentry",		sizeof(struct dentry) },
 	{ "dev_t",		sizeof(dev_t) },
 	{ "device",		sizeof(struct device) },
+	{ "exception_table_entry", sizeof(struct exception_table_entry) },
 #if ELF_CLASS == ELFCLASS32
 	{ "Elf32_Dyn",		sizeof(Elf32_Dyn) },
 	{ "Elf32_Ehdr",		sizeof(Elf32_Ehdr) },
@@ -118,6 +119,7 @@ struct sizetbl sizetbl[] = {
 	{ "vm_area_struct",	sizeof(struct vm_area_struct) },
 	{ "vm_struct",		sizeof(struct vm_struct) },
 	{ "wait_queue_head_t",	sizeof(wait_queue_head_t) },
+	{ "wait_queue_t",	sizeof(wait_queue_t) },
 	{ "zone",		sizeof(struct zone) },
 };
 int sizetbl_size = sizeof(sizetbl) / sizeof(sizetbl[0]);
