@@ -10,6 +10,7 @@ PRIVATE addr_t dmesg();
 const commandtable_t command_dmesg =
 	{"dmesg", dmesg, "", "print the kernel message ring buffer"};
 
+#if !defined(CONFIG_LOG_BUF_SHIFT) || (CONFIG_LOG_BUF_SHIFT == 0)
 #if defined(CONFIG_MULTIQUAD) || defined(CONFIG_IA64)
 #define LOG_BUF_LEN	(65536)
 #elif defined(CONFIG_ARCH_S390)
@@ -19,6 +20,9 @@ const commandtable_t command_dmesg =
 #else
 #define LOG_BUF_LEN	(16384)
 #endif
+#else /* CONFIG_LOG_BUF_SHIFT */
+#define LOG_BUF_LEN (1 << CONFIG_LOG_BUF_SHIFT)
+#endif
 
 addr_t log_buf_addr;
 addr_t log_end_addr;
@@ -39,6 +43,10 @@ dmesg()
 
 	GETADDR(log_buf);
 	GETADDR(log_end);
+#if LINUXVER>=26
+	memread(log_buf_addr, sizeof(log_buf_addr), &log_buf_addr, "log_buf");
+	/* log_buf_len is variable in Linux-2.6 */ /*XXX*/
+#endif
 	memread(log_buf_addr, LOG_BUF_LEN, buf, "log_buf");
 	memread(log_end_addr, sizeof(log_end), &log_end, "log_end");
 
