@@ -10,6 +10,9 @@
 #include <xfs_trans.h>
 #include <xfs_buf_item.h>
 #include <xfs_inode_item.h>
+#include <xfs_extfree_item.h>
+
+#include "flags_trans.h"
 
 void
 prhead_xfs_log_item()
@@ -76,6 +79,13 @@ print_xfs_log_item(addr, full)
 			mprintf("\n");
 			mprintf("  recur:    %x\n", bli.bli_recur);
 			mprintf("  refcount: %x\n", ATOMIC_READ(bli.bli_refcount));
+#if 0
+			mprintf("  format.size:     %x\n", bli.bli_format.blf_size);
+			mprintf("  format.flags:    %x\n", bli.bli_format.blf_flags);
+			mprintf("  format.len:      %x\n", bli.bli_format.blf_len);
+			mprintf("  format.blkno:    %llx\n", bli.bli_format.blf_blkno);
+			mprintf("  format.map_size: %llx\n", bli.bli_format.blf_map_size);
+#endif
 			break;
 		}
 		case XFS_LI_INODE: {
@@ -87,10 +97,36 @@ print_xfs_log_item(addr, full)
 			};
 			xfs_inode_log_item_t ili;
 			memread(addr, sizeof(ili), &ili, "xfs_inode_log_item");
-			mprintf("  inode: " FPTR "\n", ili.ili_inode);
-			mprintf("  flags:  %x ", ili.ili_flags);
+			mprintf("  inode:       " FPTR "\n", ili.ili_inode);
+			mprintf("  flags:       %x ", ili.ili_flags);
 			mprintbit(iliflags, ili.ili_flags);
 			mprintf("\n");
+			mprintf("  extents_buf: " FPTR "\n", ili.ili_extents_buf);
+			break;
+		}
+		case XFS_LI_EFI: {
+			static const struct bitname efiflags[] = {
+				{ XFS_EFI_RECOVERED,	"recovered" },
+				{ XFS_EFI_COMMITTED,	"committed" },
+				{ XFS_EFI_CANCELED,	"canceled" },
+				{ 0,			NULL }
+			};
+			xfs_efi_log_item_t efi;
+			memread(addr, sizeof(efi), &efi, "xfs_efi_log_item");
+			mprintf("  flags:       %x ", efi.efi_flags);
+			mprintbit(efiflags, efi.efi_flags);
+			mprintf("\n");
+			mprintf("  next_extent: %x\n", efi.efi_next_extent);
+#if 0
+			mprintf("  format.size: %x\n", efi.efi_format.efi_size);
+#endif
+			break;
+		}
+		case XFS_LI_EFD: {
+			xfs_efd_log_item_t efd;
+			memread(addr, sizeof(efd), &efd, "xfs_efd_log_item");
+			mprintf("  efip: " FPTR "\n", efd.efd_efip);
+			mprintf("  next_extent: %x\n", efd.efd_next_extent);
 			break;
 		}
 		}
@@ -147,9 +183,16 @@ xfs_transaction_type(t)
 	}
 }
 
+void
+prhead_xfs_trans()
+{
+	mprintf(SPTR " MAGIC   RES COUNT BLK_R             LSN ITEM     BUSY TYPE\n", "ADDR");
+}
+
 addr_t
-print_xfs_trans(addr)
+print_xfs_trans(addr, aflag)
 	addr_t addr;
+	int aflag;
 {
 	struct xfs_trans trans;
 	int i;
@@ -164,15 +207,26 @@ print_xfs_trans(addr)
 	};
 
 	memread(addr, sizeof(trans), &trans, "xfs_trans_t");
-	mprintf("addr:         " FPTR "\n", addr);
 
+	if (aflag) {
+		mprintf(FPTR "  ", addr);
+		for (i = 0; i < sizeof(trans.t_magic); i++) {
+			int c = (trans.t_magic >> (sizeof(trans.t_magic) - i - 1) * 8) & 255;
+			mprintf("%c", (c >= 0x20 && c < 0x7f)? c: '.');
+		}
+		mprintf(" %5x %5x %5x", trans.t_log_res, trans.t_log_count, trans.t_blk_res);
+		mprintf(" %15llx", (__s64)trans.t_lsn);
+		mprintf(" %4x %8x %s\n", 
+			trans.t_items.lic_free, trans.t_busy.lbc_free,
+			xfs_transaction_type(trans.t_type));
+		goto out;
+	}
+
+	mprintf("addr:         " FPTR "\n", addr);
 	mprintf("magic:        %x  ", trans.t_magic);
 	for (i = 0; i < sizeof(trans.t_magic); i++) {
-		int c = ((unsigned char *)&trans.t_magic)[i];
-		if (c >= 0x20 && c < 0x7f)
-			mprintf("%c", c);
-		else
-			mprintf(".");
+		int c = (trans.t_magic >> (sizeof(trans.t_magic) - i - 1) * 8) & 255;
+		mprintf("%c", (c >= 0x20 && c < 0x7f)? c: '.');
 	}
 	mprintf("\n");
 
@@ -192,6 +246,9 @@ print_xfs_trans(addr)
 	mprintf("ticket:       " FPTR "  (xlog_ticket)\n", trans.t_ticket);
 	/* mprintf("sema:         \n");	XXX*/
 	mprintf("lsn:          %llx\n", (__s64)trans.t_lsn);
+#ifdef _xfs_trans_has_t_commit_lsn
+	mprintf("commit_lsn:   %llx\n", (__s64)trans.t_commit_lsn);
+#endif /*_xfs_trans_has_t_commit_lsn*/
 	mprintf("mountp:       " FPTR "  (xfs_mount)\n", trans.t_mountp);
 	mprintf("dqinfo:       " FPTR "\n", trans.t_dqinfo);
 	mprintf("callback:     " FPTR "\n", trans.t_callback);
@@ -246,5 +303,9 @@ print_xfs_trans(addr)
 			trans.t_busy.lbc_busy[i].lbc_ag,
 			trans.t_busy.lbc_busy[i].lbc_idx);
 	}
-	return 0;
+
+ out:
+	if (trans.t_logcb.cb_next == 0)
+		return 0;
+	return (addr_t)trans.t_logcb.cb_next - OFFSET(struct xfs_trans, t_logcb.cb_next);
 }
